/*
 * client_config.hpp
 *
 * Configuration for a client. All config information is actually received over
 * the network by the server.
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef CLIENT_CONFIG_HPP
#define CLIENT_CONFIG_HPP

#include "boost/scoped_ptr.hpp"
using namespace boost;

#include <iosfwd>
#include <vector>
using namespace std;

class Graphic;
class Anim;
class Overlay;
class Control;
class PotionRenderer;
class SkullRenderer;

class ClientConfig {
public:
	// ctor loads the output of GameConfig::serializeAll.
	explicit ClientConfig(istream &str);

	// dtor: Deletes all the created objects.
	~ClientConfig();
	
	// Lookup functions -- convert IDs to pointers.
	Graphic * getGraphic(int id) const;
	Anim * getAnim(int id) const;
	Overlay * getOverlay(int id) const;
	Control * getControl(int id) const;

	// Other lookup functions
	PotionRenderer *getPotionRenderer() const { return potion_renderer.get(); }
	SkullRenderer *getSkullRenderer() const { return skull_renderer.get(); }
	const Graphic *getMenuCentre() const { return menu_centre; }
	const Graphic *getMenuEmpty() const { return menu_empty; }
	const Graphic *getMenuHighlight() const { return menu_highlight; }
	
private:
	vector<Graphic*> graphics;
	vector<Anim*> anims;
	vector<Overlay*> overlays;
	vector<Control*> controls;
	scoped_ptr<PotionRenderer> potion_renderer;
	scoped_ptr<SkullRenderer> skull_renderer;
	Graphic *menu_centre, *menu_empty, *menu_highlight;
};

#endif
