/*
 * loading_screen.cpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "cfg.hpp"
#include "client_config.hpp"
#include "loading_screen.hpp"
#include "menu_screen.hpp"
#include "display.hpp"
#include "sound_manager.hpp"
#include "rstream/rstream.hpp"
#include "server_config.hpp"
#include "x_centre.hpp"

namespace {
	LoadingScreen *lscr = 0;
}

void SetupLoadingScreen(const UsefulStuff &u)
{
	ASSERT(!lscr);
	lscr = new LoadingScreen(u);
	lscr->render();
	u.app2d->addObserver(lscr);
	u.app2d->setUpdateIntervalActive(50);
	u.app2d->setUpdateIntervalInactive(50);
	u.app2d->setRenderFast(false);
}

void StopLoadingScreen(const UsefulStuff &u)
{
	if (!lscr) return;
	u.app2d->rmObserver(lscr);
	delete lscr;
	lscr = 0;
}

namespace {
	void SoundCallback(const string &name, void *cxt)
	{
		SoundManager::instance().loadSound(name);
	}
	
	shared_ptr<ServerConfig> LoadServerConfig()
	{
		shared_ptr<ServerConfig> server_config(new ServerConfig);
		server_config->loadConfigFile("main.txt", &SoundCallback, 0);
		return server_config;
	}
	
	void SendServerConfig(ostream &remote, shared_ptr<ServerConfig> server_config)
	{
		server_config->serializeAll(remote);
		remote.flush();
	}

	shared_ptr<ClientConfig> LoadClientConfig(istream &remote)
	{
		if (remote.rdbuf()->in_avail()==0) {
			return shared_ptr<ClientConfig>();
		} else {
			shared_ptr<ClientConfig> client_config(new ClientConfig(remote));
			return client_config;
		}
	}
}

void LoadingScreen::render()
{
    us.display->setFontSize(CfgInt("font_size"));
	us.display->begin();
	XCentre(*us.display, us.display->getTextHeight() + 15, "LOADING");
	us.display->end();
}

void LoadingScreen::update()
{
	if (us.server) {
		us.server_config = LoadServerConfig();
		if (us.remote) {
			SendServerConfig(*us.remote, us.server_config);
		}
	} else {
		ASSERT(us.remote);
		us.client_config = LoadClientConfig(*us.remote);
		if (!us.client_config) {
			// The client config has not yet been loaded. We should wait a bit and then
			// retry (at next update point).
			return;
		}
	}

	// If we get to here then we are ready to proceed to the MenuScreen....
	SetupMenuScreen(us);
	StopLoadingScreen(us);
}
