/*
 * menu_constraint.cpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "kconfig/kfile.hpp"
#include "menu_constraint.hpp"
#include "menu_selections.hpp"
#include "server_config.hpp"

//
// Constrain
//

class Constrain : public MenuConstraint {
public:
	Constrain(const string &id_, const MenuInt * val_) : id(id_), val(val_) { }
	virtual bool apply(MenuSelections &msel);
private:
	string id;
	const MenuInt *val;
};

bool Constrain::apply(MenuSelections &msel)
{
	if (!val) return false;
	int new_val = val->getValue(msel);
	if (msel.get(id) == new_val) return false;
	msel.set(id, new_val);
	return true;
}

//
// ConstrainMin
//

class ConstrainMin : public MenuConstraint {
public:
	ConstrainMin(const string &id_, const MenuInt * val_) : id(id_), val(val_) { }
	virtual bool apply(MenuSelections &msel);
private:
	string id;
	const MenuInt *val;
};

bool ConstrainMin::apply(MenuSelections &msel)
{
	if (!val) return false;
	int mn = val->getValue(msel);
	int cur = msel.get(id);
	if (cur < mn) {
		msel.set(id, mn);
		return true;
	} else {
		return false;
	}
}
	
//
// ConstrainMax
//

class ConstrainMax : public MenuConstraint {
public:
	ConstrainMax(const string &id_, const MenuInt * val_) : id(id_), val(val_) { }
	virtual bool apply(MenuSelections &msel);
private:
	string id;
	const MenuInt *val;
};

bool ConstrainMax::apply(MenuSelections &msel)
{
	if (!val) return false;
	int mx = val->getValue(msel);
	int cur = msel.get(id);
	if (cur > mx) {
		msel.set(id, mx);
		return true;
	} else {
		return false;
	}
}

//
// Create function
//

shared_ptr<MenuConstraint> CreateMenuConstraint(const string &name, ServerConfig &sc)
{
	shared_ptr<MenuConstraint> result;

	KFile::List lst(*sc.getKFile(), "", 2);
	lst.push(0);
	string id = sc.getKFile()->popString();
	lst.push(1);
	const MenuInt * mi = sc.popMenuInt();
	
	if (name == "Constrain") {
		result.reset(new Constrain(id, mi));
	} else if (name == "ConstrainMin") {
		result.reset(new ConstrainMin(id, mi));
	} else if (name == "ConstrainMax") {
		result.reset(new ConstrainMax(id, mi));
	} else {
		sc.getKFile()->errExpected("MenuDirective");
	}
	
	return result;
}
