/*
 * sdl_font.cpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "find_font_names.hpp"
#include "my_exceptions.hpp"
#include "rstream/rstream_rwops.hpp"
#include "sdl_bitmap_font.hpp"
#include "sdl_font.hpp"
#include "sdl_truetype_font.hpp"

#include <string>
using namespace std;

SDLFont * LoadFirstAvailableFont(int size)
{
    // Try the outline fonts first.
    const int num_outline_fonts = GetNumOutlineFonts();
    for (int i = 0; i < num_outline_fonts; ++i) {
        try {
            return new SDLTrueTypeFont(GetOutlineFontName(i), size);
        } catch (...) { }
    }

    // Now try the bitmap fonts
    const int num_bitmap_fonts = GetNumBitmapFonts();
    for (int i = 0; i < num_bitmap_fonts; ++i) {
        try {
            SDL_Surface * font_surface = SDL_LoadBMP_RW(RWFromRStream(GetBitmapFontName(i)), true);
            if (font_surface) {
                return new SDLBitmapFont(font_surface);
            }
        } catch (...) { }
    }

    // Couldn't load ANY of the available fonts...
    throw InitError("Problem loading the font. Please check knights_data/fonts.txt");
}
