/*
 * sdl_graphic.hpp
 *
 * SDL implementation of Graphic, CreateGraphic and DestroyGraphic
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef SDL_GRAPHIC_HPP
#define SDL_GRAPHIC_HPP

#include "graphic.hpp"

#include <SDL.h>

class Graphic {
public:
    // same parameters as CreateGraphic
	Graphic(const string &filename, shared_array<RGB> pixels, int width, int height, int hx, int hy, int r, int g, int b);
	
	// dtor (unloads the surface)
	~Graphic();

	// blit to screen at a certain position
	// (this is used to implement Display::drawGraphic)
	void blit(SDL_Surface *dest, int sdl_x, int sdl_y) const;

	int getWidth() const { return width; }
	int getHeight() const { return height; }
    shared_array<RGB> getPixels() const { return pixels; }
	int getID() const { return id; }
    int getHX() const { return hx; }
    int getHY() const { return hy; }
    int getR() const { return r; }
    int getG() const { return g; }
    int getB() const { return b; }

	friend void SerializeGraphic(ostream &str, Graphic &g, int id);
	friend Graphic * CreateGraphicFromStream(istream &);
	
private:
	void load() const;
	void unload() const;

private:
	mutable SDL_Surface *surface;

    string filename;

    shared_array<RGB> pixels;
    int width, height;

	int hx, hy, r, g, b;
	int id;
	mutable bool need_reload;
};

#endif

