/*
 * sdl_pixels.cpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "sdl_pixels.hpp"

#include <SDL.h>

void PlotPixel(SDL_Surface *surf, int x, int y, int R, int G, int B)
{
	// The following is taken from the SDL introduction,
	// http://www.libsdl.org/intro.en/usingvideo.html
	// (but note bugfix for 24bpp mode!)

	if (x < 0 || y < 0 || x >= surf->w || y >= surf->h) return;
	
	Uint32 color = SDL_MapRGB(surf->format, R, G, B);	
	
	switch (surf->format->BytesPerPixel) {
        case 1: { /* Assuming 8-bpp */
            Uint8 *bufp;

            bufp = (Uint8 *)surf->pixels + y*surf->pitch + x;
            *bufp = color;
        }
        break;

        case 2: { /* Probably 15-bpp or 16-bpp */
            Uint16 *bufp;

            bufp = (Uint16 *)surf->pixels + y*surf->pitch/2 + x;
            *bufp = color;
        }
        break;

        case 3: { /* Slow 24-bpp mode, usually not used */
            Uint8 *bufp;

            bufp = (Uint8 *)surf->pixels + y*surf->pitch + x*3;
            *(bufp+surf->format->Rshift/8) = R;
            *(bufp+surf->format->Gshift/8) = G;
            *(bufp+surf->format->Bshift/8) = B;
        }
        break;

        case 4: { /* Probably 32-bpp */
            Uint32 *bufp;

            bufp = (Uint32 *)surf->pixels + y*surf->pitch/4 + x;
            *bufp = color;
        }
        break;
    }
}

void ReadPixel(SDL_Surface *surf, int x, int y, int &r, int &g, int &b)
{
	// this is a straightforward adaptation from PlotPixel.

	Uint8 rr, gg, bb;
	
	switch (surf->format->BytesPerPixel) {
	case 1:
		{
			Uint8 *bufp = (Uint8*)surf->pixels + y*surf->pitch + x;
			SDL_GetRGB(*bufp, surf->format, &rr, &gg, &bb);
		}
		break;

	case 2:
		{
			Uint16 *bufp = (Uint16*)surf->pixels + y*surf->pitch/2 + x;
			SDL_GetRGB(*bufp, surf->format, &rr, &gg, &bb);
		}
		break;

	case 3:
		{
			Uint8 *bufp = (Uint8*)surf->pixels + y*surf->pitch + x*3;
			rr = *(bufp+surf->format->Rshift/8);
			gg = *(bufp+surf->format->Gshift/8);
			bb = *(bufp+surf->format->Bshift/8);
		}
		break;

	case 4:
		{
			Uint32 *bufp = (Uint32*)surf->pixels + y*surf->pitch/4 + x;
			SDL_GetRGB(*bufp, surf->format, &rr, &gg, &bb);
		}
		break;
	}
	r = rr;
	g = gg;
	b = bb;
}
